# 
# Copyright (C) 2007 LAAS/CNRS 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 

%module gdheTriangle
%include "typemaps.i"

%{
#include <GL/gl.h>
#include "gdheTriangle.h"
  typedef int Tstatus;

  typedef struct structListTcl {
    Tcl_Obj** listObj;
    int nbItem;
    Tcl_Interp *interp;
  } structListTcl;
  
  extern int gdheTriangleVertexResize(gdheTriangle* pT, int nbVertex);
  extern int gdheTriangleNormalResize(gdheTriangle* pT, int nbNormal);
  extern int gdheTriangleColorResize(gdheTriangle* pT, int nbColor);
  extern int gdheTriangleTriangleResize(gdheTriangle* pT, int nbTriangle);

  extern void gdheTrianglePrint(gdheTriangle* pT);

  Tstatus gdheTriangleVertexAdd(gdheTriangle* pT, structListTcl liste[])
  {
    int i;
    GLfloat* pVertex;
    double dtmp;
    
    if (pT == 0) {
      Tcl_SetResult(liste->interp, "Pb: pt=0", TCL_STATIC);
      return -1;
    }
    if ((liste->nbItem % 3) != 0) {
      Tcl_SetResult(liste->interp, "Pb: number of elements isn't 3xN",
		    TCL_STATIC);
      return -1;
    }
    if (gdheTriangleVertexResize(pT, pT->vertexNumber + liste->nbItem) != 0) {
      Tcl_SetResult(liste->interp, "Pb: gdheTriangle: allocation problem\n",
		    TCL_STATIC);
      return -1;
    }
    pVertex = &(pT->vertex[pT->vertexNumber]);
    for (i = 0; i < liste->nbItem; i++) {
      if (Tcl_GetDoubleFromObj(liste->interp, liste->listObj[i], &dtmp)
	  == TCL_ERROR) {
	return -1;
      }
      *pVertex = (GLfloat) dtmp;
      pVertex++;
    }
    pT->vertexNumber += liste->nbItem;
    return 0;
  }

  Tstatus gdheTriangleTriangleAdd(gdheTriangle* pT, structListTcl liste[])
  {
    int i;
    GLuint* pint;
    int itmp;
    
    if (pT == 0) {
      Tcl_SetResult(liste->interp, "Pb: pt=0", TCL_STATIC);
      return -1;
    }
    if ((liste->nbItem % 3) != 0) {
      Tcl_SetResult(liste->interp, "Pb: number of elements isn't 3xN",
		    TCL_STATIC);
      return -1;
    }
    if (gdheTriangleTriangleResize(pT, pT->triangleNumber + liste->nbItem)
	!= 0) {
      Tcl_SetResult(liste->interp, "Pb: gdheTriangle: allocation problem\n",
		    TCL_STATIC);
      return -1;
    }

    pint = &(pT->triangle[pT->triangleNumber]);
    for (i = 0; i < liste->nbItem; i++) {
      // not perfect as we want GLuint : get long and copie then 
      // verify if it is < max ...
      if (Tcl_GetIntFromObj(liste->interp, liste->listObj[i], &itmp)
	  == TCL_ERROR) {
	return -1;
      }
      if (itmp < 0) {
	Tcl_SetResult(liste->interp, "Try to set negatif value into unsigned",
		      TCL_STATIC);
	return -1;
      }
      if (itmp > pT->vertexNumber -1) {
	Tcl_SetResult(liste->interp,
		      "One vertex number is greater than vertexNumber",
		      TCL_STATIC);
	return -1;
      }
      *pint = (GLfloat) itmp;
      pint++;
    }
    pT->triangleNumber += liste->nbItem;
    return 0;
  }

  Tstatus gdheTriangleNormalAdd(gdheTriangle* pT, structListTcl liste[])
  {
    int i;
    GLfloat* pNormal;
    double dtmp;
    
    if (pT == 0) {
      Tcl_SetResult(liste->interp, "Pb: pt=0", TCL_STATIC);
      return -1;
    }
    if ((liste->nbItem % 3) != 0) {
      Tcl_SetResult(liste->interp, "Pb: number of elements isn't 3xN",
		    TCL_STATIC);
      return -1;
    }
    if (gdheTriangleNormalResize(pT, pT->normalNumber + liste->nbItem) != 0) {
      Tcl_SetResult(liste->interp, "Pb: gdheTriangle: allocation problem\n",
		    TCL_STATIC); return -1;
      return -1;
    }
    pNormal = &(pT->normal[pT->normalNumber]);
    for (i = 0; i < liste->nbItem; i++) {
      if (Tcl_GetDoubleFromObj(liste->interp, liste->listObj[i], &dtmp)
	  == TCL_ERROR) {
	return -1;
      }
      *pNormal = (GLfloat) dtmp;
      pNormal++;
    }
    pT->normalNumber += liste->nbItem;
    return 0;
  }

  Tstatus gdheTriangleColorAdd(gdheTriangle* pT, structListTcl liste[])
  {
    int i;
    GLubyte* pColor;
    int btmp;
    
    if (pT == 0) {
      Tcl_SetResult(liste->interp, "Pb: pt=0", TCL_STATIC);
      return -1;
    }
    if ((liste->nbItem % 3) != 0) {
      Tcl_SetResult(liste->interp, "Pb: number of elements isn't 3xN",
		    TCL_STATIC);
      return -1;
    }
    if (gdheTriangleColorResize(pT, pT->colorNumber + liste->nbItem) != 0) {
      Tcl_SetResult(liste->interp, "Pb: gdheTriangle: allocation problem\n",
		    TCL_STATIC); return -1;
      return -1;
    }
    pColor = &(pT->color[pT->colorNumber]);
    for (i = 0; i < liste->nbItem; i++) {
      if (Tcl_GetIntFromObj(liste->interp, liste->listObj[i], &btmp)
	  == TCL_ERROR) {
	return -1;
      }
      if (btmp < 0 || btmp > 255) {
	Tcl_SetResult(liste->interp,
		      "a byte must be in [0 255]\n", TCL_STATIC); return -1;
	return -1;
      }
      *pColor = (GLubyte) btmp;
      pColor++;
    }
    pT->colorNumber += liste->nbItem;
    return 0;
  }

%}

%include "gdheTriangle.h"



%typemap(in) structListTcl liste[ANY] (structListTcl listw = { 0, 0}){
  if (Tcl_ListObjGetElements(interp, $input,
			     &(listw.nbItem), &(listw.listObj)) == TCL_ERROR) {
    return TCL_ERROR;
  }
  listw.interp = interp;
  $1 = &listw;
}

%typemap(out) Tstatus {
  if ($1 != 0) return TCL_ERROR;
}


Tstatus gdheTriangleVertexAdd(gdheTriangle* pT, structListTcl liste[]);
Tstatus gdheTriangleTriangleAdd(gdheTriangle* pT, structListTcl liste[]);
Tstatus gdheTriangleNormalAdd(gdheTriangle* pT, structListTcl liste[]);
Tstatus gdheTriangleColorAdd(gdheTriangle* pT, structListTcl liste[]);
void gdheTrianglePrint(gdheTriangle* pT);
