/*
 * Copyright (C) 1996-2003 LAAS/CNRS 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "config.h"
__RCSID("$LAAS$");

#include <sys/time.h>
#include <string.h>

/**
 ** Static variable holding scheduled redisplay date
 **/
static struct timeval redisplay_time = {0, 0};

/* prototypes des fonctions locales */
static int time_ge(struct timeval *t1, struct timeval *t2);
static void time_somme(struct timeval *t, long incr);

/** 
 ** True is current time is after scheduled redisplay date
 **/
int 
need_redisplay(struct timeval *current)
{

    return time_ge(current, &redisplay_time);
}

/** 
 ** Schedule a redisplay in the future 
 **/
void 
schedule_redisplay(struct timeval *now, long duree)
{
    memcpy(&redisplay_time, now, sizeof(struct timeval));
    time_somme(&redisplay_time, 2*duree);
}

/**
 ** time_difference - calcule la difference en millisecondes entre
 **                   deux dates
 **/
long
time_difference(struct timeval *t1, struct timeval *t2)
{
    long usec_diff = t1->tv_usec - t2->tv_usec, retenue = 0;

    if (usec_diff < 0) {
	usec_diff = 1000000 + usec_diff;
	retenue = 1;
    }
    return (t1->tv_sec - t2->tv_sec - retenue)*1000 + usec_diff/1000;
}

/**
 ** time ge - teste si t1 >= t2
 **/
static int
time_ge(struct timeval *t1, struct timeval *t2)
{
    if (t1->tv_sec != t2->tv_sec) {
	return(t1->tv_sec > t2->tv_sec);
    }
    return(t1->tv_usec >= t2->tv_usec);
}


/**
 ** time_somme - ajoute une dure'e en millisecondes a` t
 **/
static void
time_somme(struct timeval *t, long incr)
{
    long incr_sec = incr / 1000;
    long incr_usec = (incr % 1000) * 1000;
    long retenue = 0;
    long usec_somme = t->tv_usec + incr_usec;
    
    retenue = usec_somme / 1000000;
    t->tv_usec = usec_somme % 1000000;
    
    t->tv_sec += incr_sec + retenue;
}
